@@ -1,6 +1,6 @@
 /*
  *	C-string functions
- *	Copyright © Jan Engelhardt, 1999-2010
+ *	Copyright Jan Engelhardt, 1999-2010
  *
  *	This file is part of libHX. libHX is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU Lesser
@@ -38,7 +38,7 @@ struct HX_quote_rule {
 	const char *chars;
 };
 
-static const char HX_hexenc[] = "0123456789ABCDEF";
+static const char HX_hexenc[16] = "0123456789ABCDEF";
 
 static __inline__ unsigned int min_uint(unsigned int a, unsigned int b)
 {
@@ -48,9 +48,15 @@ static __inline__ unsigned int min_uint(unsigned int a, unsigned int b)
 EXPORT_SYMBOL char *HX_basename(const char *s)
 {
 	const char *p;
+	/* ignore trailing slashes */
 	for (p = s + strlen(s) - 1; p >= s && *p == '/'; --p)
 		;
 	if (p < s)
+		/*
+		 * String contained only slashes - this must be the root
+		 * directory. Since we have the opportunity, rather than
+		 * returning "////", just give the cleaned-up "/".
+		 */
 		return const_cast1(char *, s + strlen(s) - 1);
 	if ((p = HX_strbchr(s, p, '/')) != NULL)
 		return const_cast1(char *, p + 1);
@@ -65,9 +71,11 @@ EXPORT_SYMBOL char *HX_basename_exact(const char *s)
 
 	if (*s == '\0')
 		return HX_strdup(".");
+	/* ignore trailing slashes */
 	for (end = s + strlen(s) - 1; end >= s && *end == '/'; --end)
 		;
 	if (end < s)
+		/* string consisted of only slashes */
 		return HX_strdup("/");
 
 	start = HX_strbchr(s, end, '/');
@@ -357,13 +365,20 @@ EXPORT_SYMBOL size_t HX_strltrim(char *expr)
 	size_t diff = 0;
 	travp = expr;
 
-	while (*travp != '\0' && HX_isspace(*travp))
+	while (HX_isspace(*travp))
 		++travp;
 	if ((diff = travp - expr) > 0)
-		memmove(expr, travp, diff);
+		memmove(expr, travp, strlen(travp) + 1);
 	return diff;
 }
 
+EXPORT_SYMBOL char *HX_stpltrim(const char *p)
+{
+	while (HX_isspace(*p))
+		++p;
+	return const_cast1(char *, p);
+}
+
 /* supports negative offsets like scripting languages */
 EXPORT_SYMBOL char *HX_strmid(const char *expr, long offset, long length)
 {
@@ -489,6 +504,7 @@ static const struct HX_quote_rule HX_quote_rules[] = {
 	[HXQUOTE_LDAPFLT] = {HXQUOTE_REJECT, "\n*()\\"},
 	[HXQUOTE_LDAPRDN] = {HXQUOTE_REJECT, "\n \"#+,;<=>\\"},
 	[HXQUOTE_URIENC]  = {HXQUOTE_ACCEPT, "-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz~"},
+	[HXQUOTE_SQLSQUOTE] = {HXQUOTE_REJECT, "'"},
 };
 
 /**
@@ -558,6 +574,30 @@ static char *HX_quote_backslash(char *dest, const char *src, const char *qc)
 	return ret;
 }
 
+static char *
+HX_quote_sqlbackslash(char *dest, const char *src, const char *trm)
+{
+	char *ret = dest;
+	size_t len;
+
+	while (*src != '\0') {
+		len = strcspn(src, trm);
+		if (len > 0) {
+			memcpy(dest, src, len);
+			dest += len;
+			src  += len;
+			if (*src == '\0')
+				break;
+		}
+		*dest++ = *trm;
+		*dest++ = *trm;
+		++src;
+	}
+
+	*dest = '\0';
+	return ret;
+}
+
 /**
  * Encode @src into BASE-64 according to RFC 4648 and write result to @dest,
  * which must be of appropriate size, plus one for a trailing NUL.
@@ -709,6 +749,7 @@ static size_t HX_quoted_size(const char *s, unsigned int type)
 	switch (type) {
 	case HXQUOTE_SQUOTE:
 	case HXQUOTE_DQUOTE:
+	case HXQUOTE_SQLSQUOTE:
 		return HX_qsize_bsr(s, HX_quote_rules[type].chars, 1);
 	case HXQUOTE_HTML:
 		return HX_qsize_html(s);
@@ -777,6 +818,8 @@ EXPORT_SYMBOL char *HX_strquote(const char *src, unsigned int type,
 		return HX_quote_base64(*free_me, src);
 	case HXQUOTE_URIENC:
 		return HX_quote_urlenc(*free_me, src);
+	case HXQUOTE_SQLSQUOTE:
+		return HX_quote_sqlbackslash(*free_me, src, rule->chars);
 	}
 	return NULL;
 }
